{{
***************************************************
* LED driver v2.05   (1wire version)              *
* NEEDS DEMO to run, always compile from Demo tab *
* Author: Tony Philipsson                         *
* Copyright 2011 Electrons Engineering            *
***************************************************
}}
CON   
    
    CLK = 0               'pin for CLK(Serial data)   0=P0, 1=P1 etc
 
    A1=  %1<<1             'the 17 LED Segments
    A2=  %1<<2
    B=   %1<<3
    C=   %1<<5
    D1=  %1<<6
    D2=  %1<<7
    E=   %1<<9
    F=   %1<<10
    G1=  %1<<21
    G2=  %1<<15
    H=   %1<<11
    I=   %1<<13
    J=   %1<<14
    K=   %1<<17
    L=   %1<<18
    M=   %1<<19
    DP=  %1<<22

PUB Display (text)
    delay1 := clkfreq/1_000_000 
    delay15 := clkfreq/1_000_000 * 15
    delay30 := clkfreq/1_000_000 * 30
    delay200 := clkfreq/1_000_000 * 200
    delay300 := clkfreq/1_000_000 * 300
    cognew(@asm_entry, text)   'launch assembly program in a COG    

DAT
              org  0

asm_entry     mov       dira,CLKpin                     'make only clk pin an output
              mov       outa,CLKpin                     'set all pins in this cog to low but CLKpin high 
main          rdlong    sysclk,#0                       'get System Hz (how many ticks in 1sec)
              mov       cnt,sysclk                      'in case we will wait 1sec
              shr       sysclk,#23 wz                   'mimimum speed we can handle is around 8mhz
        if_nz jmp       #fastenough
              add       cnt,cnt                         'add cnt to shadow cnt.
              waitcnt   cnt,#0                          'wait 1 second
              xor       outa,CLKpin                     'send a 1sec pulse
              jmp       #main                           'jump back and try again.         
fastenough    mov       digit,#6                        'we have 6 digits to multiplex
              mov       bytepnt,#0                      'reset the byte-from-a-long-decoder.
              mov       sink, #1                        'reset transistor sink 
              mov       buffer,par                      'reset buffer (par=hub address of first letter)
              rdlong    mytext,buffer                   'get first 4 bytes
              add       buffer,#4
              rdlong    mytext+1,buffer                 'get the last 4 bytes.
           

loop1         call      #getbyte                        'look up next char
        if_z  mov       mychar,#32                      'if zero make it a space char(32).
        if_nz add       bytepnt,#1                      'any other char, prepare next char          
              cmp       mychar,#123 wc                  'check if it's char({) or higher
        if_nc add       mychar,#6                       'adjust for the sub 32 below
              cmp       mychar,#97 wc                   'check if it's char(a) or higher       
        if_nc sub       mychar,#32                      'our font table have a-z omitted.                       '                       
              sub       mychar,#24                      'subtract as our ascii table start at char(24)
              
              movs      fontpnt,#font                   'reset font pointer
              add       fontpnt,mychar                  'add ascii code that mytext have, to font pointer
              mov       bit_test,bit23                  'reset bit mask so bit 23 is only set
fontpnt       mov       serial,0-0                      'start a new serial data       
              or        serial,sink                     'fuse in the transistor sink bit

              call      #getbyte                        'check next char comming up. 
              cmp       mychar,#46 wz                   'test if the next char is a dot
        if_z  or        serial,mydot                    'merge in a dot to current char.
        if_z  add       bytepnt,#1                      'a dot don't gets it's own led digit.
              
                                                        'inner loop starts here, send 23 clk pulses plus a long latch
loop2         test      serial,bit_test wz              'test serial data bitwize, set z                                
        if_nz mov       cnt,delay1                      'a quick clk, move 1uS to Shadow Cnt.
        if_z  mov       cnt,delay15                     'a long clk (15uS), let SER dip low
              andn      outa,CLKpin                     'set pin low 
              add       cnt,cnt                         'add current cnt to shadow cnt.
        if_nz waitcnt   cnt, delay15                    'wait 1uS, add 15us to cnt when done.
        if_z  waitcnt   cnt, delay30                    'wait 15us, add 30ms to cnt when done
              or        outa,CLKpin                     'set pin high 
              waitcnt   cnt, #0                         'wait the 15us or 30us that was set above
              shr       bit_test,#1 wz                  'shift right
        if_nz jmp       #loop2                          'if not zero, jmp 
         
              mov       cnt,delay200                    'prepare a 200us delay, letting Latch go low.    
              andn      outa,CLKpin                     'set pin low  
              add       cnt, cnt                        'add cnt to shadow cnt     
              waitcnt   cnt, delay300                   'wait 200us, add 300us to cnt when done. 
              or        outa,CLKpin                     'set pin high. Add any of YOUR CODE between this....
              waitcnt   cnt, #0                         'and this line, change delay300 up to max 1800 if needed.
              shl       sink,#4                         'next digit
              djnz      digit, #loop1                   'have we done 6 digits?

              jmp       #main                           'restart

             '*** sub routine byte-from-a-long-decoder ***
getbyte       mov bytepntbuf, bytepnt                   'don't trash bytepnt
              ror bytepntbuf,#2                         'divide by 4 but roll lower two bits to top.
              movs findbyte,bytepntbuf                  'initiate the start location
              add findbyte,#mytext                      'add the start location of our byte table
              shr bytepntbuf,#27                        'shift the upper two bits down 
              mov bytepntbuf, bytepnt                   'start with a fresh bytepointer
              and bytepntbuf,#%11                       'keep only the 2 lower bits
              shl bytepntbuf,#3                         'multiply by 8
findbyte      mov mychar,0-0
              shr mychar,bytepntbuf                     'find the right byte from a long
              and mychar,#$ff wz                        'keep only the lower 8 bits, set z
getbyte_ret   ret                                       'return, with answer in mybyte (also with z set)

              
              
font          long      A2+D1+D2+G1+G2+I+L              'Pund , start our font at Char(24)
              long      G1+G2+H+J+L                     'Yen
              long      A1+A2+D1+D2+E+F+G1+G2+I+L       'Euro
              long      A1+F+G1+I                       'degrees
              long      E+F+H+J                         '
              long      D1+D2+G1+G2+I+L                 '
              long      I+K+L+M                         'arrow up 
              long      H+I+J+L                         'Arrow down
              long      0                               'Space 
              long      B+C                             '!
              long      F+I                             '"
              long      A1+A2+B+C+D1+D2+E+F+G1+G2+H+I+J+K+L+M'# (all 16 on)
              long      A1+A2+C+D1+D2+F+G1+G2+I+L       '$
              long      A1+C+D1+F+G1+G2+I+J+L+M         '%
              long      A2+C+D1+D2+I+J+L+M              '&
              long      J                               ''
              long      J+K                             '(
              long      H+M                             ')
              long      G1+G2+H+I+J+K+L+M               '*
              long      G1+G2+I+L                       '+
              long      M                               ',
              long      G1+G2                           '-
mydot         long      DP                              '.
              long      J+M                             '/
              long      A1+A2+B+C+D1+D2+E+F+J+M         '0
              long      B+C                             '1
              long      A1+A2+B+D1+D2+E+G1+G2           '2
              long      A1+A2+B+C+D1+D2+G2              '3
              long      B+C+F+G1+G2                     '4
              long      A1+A2+D1+D2+F+G1+K              '5
              long      A1+A2+C+D1+D2+E+F+G1+G2         '6
              long      A1+A2+B+C                       '7
              long      A1+A2+B+C+D1+D2+E+F+G1+G2       '8
              long      A1+A2+B+C+D1+D2+F+G1+G2         '9
              long      I+L                             ':
              long      I+M                             ';
              long      J+K                             '<
              long      D1+D2+G1+G2                     '=
              long      H+M                             '>     
              long      A2+B+G2+L                       '?
              long      A1+A2+B+D1+D2+E+F+G2+I          '@
              long      A1+A2+B+C+E+F+G1+G2             'A
              long      A1+A2+B+C+D1+D2+G2+I+L          'B
              long      A1+A2+D1+D2+E+F                 'C
              long      A1+A2+B+C+D1+D2+I+L             'D
              long      A1+A2+D1+D2+E+F+G1+G2           'E
              long      A1+A2+E+F+G1                    'F
              long      A1+A2+C+D1+D2+E+F+G2            'G
              long      B+C+E+F+G1+G2                   'H
              long      I+L                             'I
              long      B+C+D1+D2+E                     'J
              long      E+F+G1+J+K                      'K
              long      D1+D2+E+F                       'L
              long      B+C+E+F+H+J                     'M
              long      B+C+E+F+H+K                     'N
              long      A1+A2+B+C+D1+D2+E+F             'O
              long      A1+A2+B+E+F+G1+G2               'P
              long      A1+A2+B+C+D1+D2+E+F+K           'Q
              long      A1+A2+B+E+F+G1+G2+K             'R
              long      A1+A2+C+D1+D2+F+G1+G2           'S
              long      A1+A2+I+L                       'T
              long      B+C+D1+D2+E+F                   'U
              long      E+F+J+M                         'V
              long      B+C+E+F+K+M                     'W
              long      H+J+K+M                         'X
              long      H+J+L                           'Y
              long      A1+A2+D1+D2+J+M                 'Z
              long      A2+D1+I+L                       '[
              long      H+K                             '\
              long      A1+D2+I+L                       ']
              long      A2+B+J+M                        '^
              long      D1+D2                           '_
              long      H                               '`
              long      A2+D1+G1+I+L                    '{
              long      I+L                             '|
              long      A1+D2+G2+I+L                    '}
              long      B+E+G1+G2                       '~
              long      A1+A2+B+C+D1+D2+E+F+G2+L        'CR
              
CLKpin        long      |<CLK                           'set 1bit to a 1 at CLK position.
bit23         long      |<23                            'same as %1<<23,  serial bits to shift out
delay1        long      0-0
delay15       long      0-0
delay30       long      0-0
delay200      long      0-0
delay300      long      0-0 
sysclk        res       1
bytepnt       res       1
bytepntbuf    res       1
sink          res       1                               'what transistor to sink
mytext        res       2                               'reserve 8 bytes (2 longs)
mychar        res       1
byteshft      res       1
buffer        res       1
bit_test      res       1
serial        res       1
digit         res       1                              